-- Foliensatz EDS-F4, Abschnitt 3.1 Operationsablaeufe. Serielle Schnittstelle
-- Datei: test_uart.vhd
-- Testrahmen fr UART-Sender und -Empfaenger
-- letzte Aenderung: 14.06.2016
-- Autor: G. Kemnitz

-- Zum Ausprobieren
-- ghdl -a uart_sender.vhd
-- ghdl -a uart_receiver.vhd
-- ghdl -a test_uart.vhd
-- ghdl -m test_uart
-- ghdl -r test_uart --wave=test_uart.ghw
-- gtkwave test_uart.ghw test_uart.sav


library IEEE;
use ieee.std_logic_1164.all;

entity test_uart is end entity;

architecture a of test_uart is
 signal dat_s, dat_r: std_logic_vector(7 downto 0);
 signal T_send, I_send, T_rec, I_rec: std_logic;
 signal start, TxD, RxD, busy_send, busy_rec: std_logic;
 constant tb_send: delay_length := 1 sec/9620;
 constant tb_rec:  delay_length := 1 sec/9513;
begin

sender: entity work.uart_sender
 port map (start=>start, T=>T_send, I=>I_send, dat=>dat_s, TxD=>TxD, busy=>busy_send);

receiver: entity work.uart_receiver 
 port map (T=>T_rec, I=>I_rec, RxD=>RxD, dat=>dat_r, busy=>busy_rec);
 
 RxD <= transport TxD after 27 us;

 process    -- init and clock generation sender
 begin
  T_send <= '0'; I_send<='1'; wait for tb_send/2;
  T_send <= '1';              wait for tb_send/2;
  T_send <= '0'; I_send<='0'; wait for tb_send/2;
  while now < 3 ms loop
   T_send <= not T_send;      wait for tb_send/2;
  end loop;
  wait;
 end process;

 process    -- init and clock generation receiver
 begin
  T_rec <= '0'; I_rec<='1'; wait for tb_rec/32;
  T_rec <= '1';             wait for tb_rec/32;
  T_rec <= '0'; I_rec<='0'; wait for tb_rec/32;
  while now < 3 ms loop
   T_rec <= not T_rec;      wait for tb_rec/32;
  end loop;
  wait;
 end process;

 InpProc: process
   begin
    start <= '0'; wait for 200 us;
    start <= '1'; dat_s <= x"D5";
    wait until busy_send = '1'; wait for 40 us;
    start <= '0';
    wait until busy_send = '0'; wait for 150 us;
    start <= '1'; dat_s <= x"6A";
    wait until busy_send = '1'; wait for 63 us;
    start <= '0';
    wait;
  end process; 

end architecture;
