-- Foliensatz EDS-F4, Abschnitt 3.1 Operationsablaeufe. Serielle Schnittstelle
-- Datei: uart_sender.vhd
-- Simulations- und Synthesebeschreibung fuer den UART-Sender
-- letzte Aenderung: 14.06.2016
-- Autor: G. Kemnitz


library IEEE;
use ieee.std_logic_1164.all;

entity uart_sender is
 port(start, T, I: in std_logic; 
  dat: in std_logic_vector(7 downto 0);
  TxD, busy: out std_logic);
end entity;

architecture a of uart_sender is
 type t_state is (s0, sd, s1);
 signal state: t_state;
 signal I_del, start_del: std_logic;
 signal ct: natural range 0 to 7;
begin

 process(T)        -- sampling
 begin
  if rising_edge(T) then
   I_del <= I;
   start_del<= start;
  end if;
 end process;
 
 process(I_del, T) -- state machine
 begin 
  if I_del='1' then
   state <= s1;              
   TxD <= '1'; busy<='0';
  elsif rising_edge(T) then
   case state is
    when s1 => TxD<='1';      -- stopbit (send 1)
     busy<='0';  ct<=0; 
     if start_del='1' then state<=s0; end if;
    when s0 => TxD<='0';     -- start bit (send 0)
     busy<='1';  ct<=0;   state<=sd; 
    when sd => TxD<=dat(ct); -- send data bit
     busy<='1'; 
     if ct=7 then state<=s1; ct <= 0;
     else ct<=ct+1;
     end if;
   end case;
  end if;
 end process;
end architecture;
